//keyboard.c              
/*
Connect the keyboard as follows

PD2(EXT INT 0) - KBD CLK
PD3 - KBD DATA

*/                           
#include <Mega163.h>
#include "keyboard.h"
//#define debug 1
//#define debug_kbd_handler 	1
//#define debug_kbd_isr		1

void initialize_keyboard(void)
begin     
//set port direction registers
//turn pull up on
//DDRD = (0x01<<2) | (0x01<<3);
//printf("DDRD=%x\n\r",DDRD);
DDRD.2 = 0;
DDRD.3 = 0;                 

PORTD.2 = 1;
PORTD.3 = 1;                

//initialize edge count
kbd_edge_count  = 0;
//initialize buffers
kbd_current_byte = 0;
kbd_hptr = 0;
kbd_tptr = 0;

//init statemachine
kbd_state = WAIT;

//init status flags
kbd_err = 0;
kbd_shift = NOSHIFT;
kbd_numlock = NONUMLOCK;
kbd_dir = RX; 

//set interrupt on falling edge
MCUCR = MCUCR | 0x02;
//just need to turn bit 0 off and on for falling and rising edge respectively
kbd_ISR_on_falling_edge

//Turn on keyboard interrupt
kbd_int_on

#ifdef debug
printf("Debug mode\n\r");
printf("DDRD=%x\n\r",DDRD);
printf("PORTD=%x\n\r",PORTD);
printf("MCUCR=%x\n\r",MCUCR);
printf("GIMSK=%x\n\r",GIMSK);
#endif

kbd_send_cmd(0xff,0x00);//reset keyboard
end        

//kbd_send_cmd()
/*Initializes the keyboard to accept a byte           
 * cmd | arg
 * ED  | bit 0 - scroll lock
 *     | but 1 - num lock
 *     | bit 2 - capslock         
 * EE  | n/a. echo EE
 * F0  | scandcode set. not used here.
 * F3  | typematic repeat rate
 * F4  | n/a. keyboard enable
 * F5  | n/a. keyboard disable
 * FE  | n/a. resend last byte
 * FF  | n/a. reset keyboard
  */   
void kbd_send_cmd(unsigned char cmd, unsigned char arg)
begin       
//prevent kbd from starting transfer, disable kbd_isr                              
kbd_int_off
//initialize the state machines and load buffers
kbd_cmd_buf[0] = cmd;
kbd_cmd_buf[1] = arg;
kbd_cmd_byte = kbd_cmd_buf[0];

switch(cmd)
  begin          
  case 0xee://echo
    kbd_state = ECHO;
    break;
  case 0xff://reset
    kbd_state = WAIT;
    break;
  case 0xed://set status LEDs
  case 0xf0://set scancode
  case 0xf3://set Typematic repeat rate
    kbd_state = ACK;
    break;
  case 0xf4://keyboard enable
  case 0xf5://keyboard disable
    kbd_state = ACK_END;
    break;         
  case 0xfe://resend
    //kbd_state unchanged
    break;
  default:
    break;
  end//switch(cmd)     

kbd_send_byte();
kbd_int_on
end               


void kbd_send_byte(void)
begin                     
//save cmd_byte
kbd_cmd_sent = kbd_cmd_byte;

//kbd_ISR_on_rising_edge
//Set both lines as output
kbd_clk_dir  = 1;
kbd_clk_port = 0;
kbd_data_dir  = 1;
delay_us(200);//100);original try 200 to see if err msg reduced, seems to reduce amt of error

//set data line low
kbd_data_port = 0;

//release kbd_clk
kbd_clk_port = 1;//pull up on
kbd_clk_dir  = 0;//output    

kbd_edge_count = 0;
kbd_dir = TX;
end         

//kbd_getch()
//Get a character from the keyboard buffer
unsigned char kbd_getch(void)
begin                
unsigned char new_hptr,temp_tptr,ascii_byte;
kbd_cts_off();//Prevent write to buffer
//must detect hptr==tptr and later turn kbd_cts_on
//Calculate next item to read
new_hptr=kbd_hptr+1;
if (new_hptr==kbd_buf_size) new_hptr=0;

temp_tptr = kbd_tptr + 1;
if (temp_tptr== kbd_buf_size) temp_tptr=0;

if (kbd_tptr==kbd_hptr)
  begin//nothing to read, buffer empty    
  #ifdef debug_kbd_getch
  printf("kbd buf empty, getch() returning\n\r");
  #endif  
  kbd_cts_on();//enable write to buffer
  return 0;
  end      
else
  begin
  ascii_byte=kbd_buf[kbd_hptr];
  kbd_hptr = new_hptr;
  end
kbd_cts_on();//enable write to buffer
#ifdef debug_kbd_getch
printf("getch() returning, %c\n\r",ascii_byte);
#endif
return ascii_byte;         
end                                                                      

#ifdef debug
#define echo 1 //echo the character pressed
#endif
//#define debug_scancode_handler 1
//#define echo_hex 1 //echo the hex value of the keypress
//scancode_handler(unsigned char data)
//data is the scancode of the key that is pressed
//this function checks for capslock, shift and numlock and
//does the appropiate action
void scancode_handler(unsigned char data)
begin  
unsigned char ascii_byte,next_tptr;
unsigned char tshift;//temp shift_status
    //scancode handler
    switch(data)
      begin      
      case 0x59://Turn shift on
      case 0x12://Turn shift on
        kbd_shift = SHIFT;
        break;      
      case 0x77://numlock
        break;
      case 0x58://capslock
        break;
      default:          
        if (kbd_shift&&!kbd_capslock) tshift = 1;
        else if (kbd_shift&&kbd_capslock) tshift = 0;
        else if (!kbd_shift&&kbd_capslock) tshift = 1;
        else tshift = 0;
        if (kbd_numlock&&tshift) 	ascii_byte = table_shift_numlock[data];
        if (!kbd_numlock&&tshift)	ascii_byte = table_shift_nonumlock[data]; 
        if (kbd_numlock&&!tshift)	ascii_byte = table_noshift_numlock[data];
        if (!kbd_numlock&&!tshift)	ascii_byte = table_noshift_nonumlock[data];
	if (ascii_byte) //non-zero, write to buffer
	  begin     
	  #ifdef echo_hex
	  printf("%x",ascii_byte);//testing whether echo EE command works
	  #endif
	  #ifdef echo
	  printf("%c",ascii_byte);
	  #endif   
	  #ifdef debug_scancode_handler
	  //printf("Writing to buffer :%c\n\r",ascii_byte);
	  #endif     
	  next_tptr = kbd_tptr+1;//next_tptr=the next tptr if char written to buffer
	  if (next_tptr==kbd_buf_size) next_tptr = 0;
	  if (kbd_hptr!=next_tptr)//if buf not full
	    begin
            kbd_buf[kbd_tptr] = ascii_byte;
	    kbd_tptr=next_tptr;
	    #ifdef debug_scancode_handler
	    printf("h=%d,t=%d,n=%d\n\r",kbd_hptr,kbd_tptr,next_tptr);
	    #endif
	    end
	  end
        break;
      end//switch data    
ext_key = 0;
//end scancode handler
end                          

//release_handler(unsigned char data)
//data is the scancode of the key most recently released
//release handler updates the shift,numlock,capslock flags accordingly
void release_handler(unsigned char data)
begin
switch(data)
  begin
  case 0x12:
  case 0x59:
    kbd_shift = NOSHIFT;
    break;
  case 0x77://Toggle numlock
    if (kbd_numlock)
      begin
      kbd_numlock = 0;//numlock off
      kbd_status_led = kbd_status_led & 0xfd;//bit1 is numlock
      kbd_send_cmd(0xed,kbd_status_led);
      end
    else
      begin
      kbd_numlock = 1;
      kbd_status_led = kbd_status_led | 0x02;
      kbd_send_cmd(0xed,kbd_status_led);
      end
    break;
  case 0x58://Toggle capslock
    if (kbd_capslock) 
      begin
      kbd_capslock = NOCAPSLOCK;
      kbd_status_led = kbd_status_led & 0xfb;//bit2 is capslock
      kbd_send_cmd(0xed,kbd_status_led);
      end
    else 
      begin
      kbd_capslock = CAPSLOCK;
      kbd_status_led = kbd_status_led | 0x04;
      kbd_send_cmd(0xed,kbd_status_led);
      end
    break;
  default:
    break;
  end//switch data 
ext_key = 0;
end
   
//This is called when a new data byte is recieved by the kbd_falling_edge ISR
void kbd_handler(unsigned char data)
begin      
#ifdef debug_kbd_handler
printf("Handler\n\r");
#endif   
switch(kbd_state)
  begin                                           
  case WAIT:
    switch(data)
      begin
      case 0xaa://power on self test passed
        kbd_state = WAIT;
        //reinitialize the status leds
        kbd_send_cmd(0xed, kbd_status_led);
        #ifdef debug
        printf("kbd self test passed\n\r");
        #endif
        return;
      case 0xfe://resend
        //resend last byte
        #ifdef debug
        printf("kbd req resend\n\r");
        #endif
        kbd_resend_tries++;
        if (kbd_resend_tries<10)
          begin
          kbd_state = WAIT; 
          #ifdef debug
          printf("resending %x, buf=%02x%02x\n\r",kbd_cmd_sent,kbd_cmd_buf[0],kbd_cmd_buf[1]);
          #endif
          kbd_cmd_byte = kbd_cmd_sent;
          kbd_int_off
          kbd_send_byte(); 
          kbd_int_on
          end
        else
          begin
          kbd_resend_tries=0;
          kbd_state = WAIT;
          kbd_send_cmd(0xff,0x00);
          end
        return;           
      case 0x00:
      case 0xff:
        //buffer overflow            
        #ifdef debug
        printf("buf ovrfl, kbd reset\n\r");
        #endif
        kbd_send_cmd(0xff,0x00);//reset keyboard
        kbd_state = WAIT;
        return;
      case 0xe0://keyboard sending extended key
        ext_key = 1;
        kbd_state = EXT_E0;
        return;
      case 0xe1://pause key depressed
        p_ptr = 1;
        kbd_state = EXT_PAUSE;       
        return;
      case 0xf0://key released
        kbd_state = RELEASE;
        return;               
      case 0xfa://ack
        #ifdef debug
        //printf("kbd ack not expected\n\r");
        #endif
        kbd_state = WAIT;
        return;
      default://Scancode handler
        kbd_state = WAIT;
        scancode_handler(data);
        return;
      end//switch(data)                                     
    return;
  case EXT_E0:
    if (data == 0xf0)
      kbd_state = RELEASE;
    else//data!=0xf0
      begin                  
      kbd_state = WAIT;
      scancode_handler(data);
      end
    return;
  case RELEASE:
    kbd_state = WAIT;
    release_handler(data);
    return;
  case EXT_PAUSE://get code sequence for pause key
    if (p_ptr>7) return;
    if (data!=pause_seq[p_ptr])
      begin//error
      //reset keyboard
      pause_key_error = 1;
      #ifdef debug
      printf("Pause key error in kbd_handler\n\r");
      #endif
      end
    p_ptr++;
    return;
  case ECHO:
    if (data!=0xee)
      begin
      //need to resend?
      #ifdef debug
      printf("Err in kbd_handler, EE expected but not recieved\n\r");
      kbd_state = WAIT;//for debug only
      #endif
      end
    else
      begin            
      kbd_state = WAIT;
      end    
    return;
    break;  
    
  case ACK://need to send a second byte
    if (data==0xfa)//ack
      begin
      //kbd_state = WAIT;                         
      kbd_state = ACK_END;//expect another ACK
      delay_us(50);//wait until kbd_releases clk
      //kbd_ISR_on_rising_edge
      //Set both lines as output
      kbd_clk_dir  = 1;
      kbd_clk_port = 0;
      kbd_data_dir  = 1;
      delay_us(100);

      //set data line low
      kbd_data_port = 0;

      //release kbd_clk before exiting kbd_handler
      kbd_dir = TX;
      kbd_cmd_byte = kbd_cmd_buf[1];
      kbd_cmd_sent = kbd_cmd_byte;
      return;
      end              
    else
      begin
      #ifdef debug
      printf("Err in kbd_handler, ACK not rcv\n\r");
      #endif      
      kbd_ack_err++;
      if (kbd_ack_err<3)
        begin
  
        //kbd_cmd_byte=kbd_cmd_sent;//resend cmd byte
        kbd_cmd_byte=kbd_cmd_buf[0]; 
        #ifdef debug                            
        printf("resending %x, buf=%02x%02x\n\r",kbd_cmd_byte,kbd_cmd_buf[0],kbd_cmd_buf[1]);
        #endif 
        kbd_int_off
        kbd_send_byte();
        kbd_int_on
        kbd_state = ACK;
        end
      else //give up trying to send cmd
        begin                          
        kbd_ack_err = 0;
        kbd_state=WAIT;
        end
      end
    return;
    break;
  case ACK_END:      
    if (data==0xfa)//ack
      begin
      kbd_state = WAIT;
      return;
      end                           
    else
      begin 
      #ifdef debug
      printf("Err in kbd_handler, ACK not rcv 2\n\r");
      #endif
      //resend last byte
      kbd_ack_err++;
      if (kbd_ack_err<3)
        begin
        kbd_cmd_byte = kbd_cmd_sent;
        kbd_int_off
        kbd_send_byte();
        kbd_int_on
        kbd_state = ACK_END;
        end
      else
        begin//give up trying to resend
        kbd_ack_err=0;
        kbd_state=WAIT;
        end
      end
    return;
    break;
  default:
    #ifdef debug
    printf("Err in kbd_handler, undef state\n\r");
    #endif
    break;
  end//switch(data)
end                

void test_kbd_handler(unsigned char data)
begin
kbd_buf[kbd_buf_idx++] = data;
kbd_edge_count=0; 
delay_ms(1000U);
end
                    
//kbd_falling_edge interrupt
//Occurs when a falling edge is detected on the
//KBD CLK line (PD.2)
//requires MCUCR bit 1 to be set (0x02).
//and GIMSK bit 6 to be set (0x40)
//There cannot be any blocking functions here,
//eg. printf, they are too slow and cause missed data
interrupt [EXT_INT0] void kbd_falling_edge(void)
begin                
unsigned char data,next_tptr;
if (kbd_dir == RX) 
  begin
  data = PIND.3;
  kbd_edge_count++;
        switch (kbd_edge_count)
          begin                 
          case 1://start bit       
            kbd_parity = 1;//if the number of 1's in data even, set parity to 1
            if (data!=0) 
              begin
              kbd_err++;
              if (kbd_err==3) 
                begin          
                kbd_err=0;
                kbd_send_cmd(0xff,0x00);//reset
                return;
                end
              else 
                begin
                kbd_send_cmd(0xfe,0x00);//resend
                return;
                end
              end
            break;                    
          case 10://parity bit
            if (data = (kbd_parity&0x01))
              begin
              //data valid
              kbd_byte_valid = 1;
              end         
            else
              begin
              kbd_err++; 
              if (kbd_err==3) 
                begin          
                kbd_err=0;
                kbd_send_cmd(0xff,0x00);//reset
                return;
                end
              else 
                begin
                kbd_send_cmd(0xfe,0x00);//resend
                return;
                end
              end
            break;
          case 11://stop bit
            if (data!=1) 
              begin
              kbd_err++;
              if (kbd_err==3) 
                begin          
                kbd_err=0;
                kbd_send_cmd(0xff,0x00);//reset
                return;
                end
              else 
                begin
                kbd_send_cmd(0xfe,0x00);//resend
                return;
                end
              end
            else //Valid byte, do data handling now
              begin                                    
              //STOP KBD entry so we do not miss bytes
              kbd_cts_off();
              #ifdef debug_kbd_isr
              printf("Turning cts off\n\r");
              #endif
              //printf("%x rx\n\r",kbd_current_byte);              
              kbd_handler(kbd_current_byte);
               
              if (kbd_dir==TX) //if we need to send second byte of kbd cmd
                begin         
                kbd_int_on
                
                kbd_edge_count = 1;//start bit sending now
                kbd_parity = 1;//init parity
            
                //release kbd_clk before exiting kbd_handler
                kbd_clk_port = 1;//pull up on
                kbd_clk_dir  = 0;//input    
                       
                return;//wait for falling edge by kbd, this isr will run again
                end
              //Completed recieving a byte from the keyboard
              kbd_edge_count=0;//prepare to recieve another byte
              next_tptr = kbd_tptr+1;
              if (next_tptr==kbd_buf_size) next_tptr=0;
              //printf("t=%x,n=%x,kbs=%x.",kbd_tptr,next_tptr,kbd_buf_size);
              if (next_tptr !=  kbd_hptr) 
                //if buffer not full...
                begin
                #ifdef debug_kbd_isr
                printf("Turning cts back on\n\r");
                #endif       
                //turn cts on so that we can receive the next byte.
                kbd_cts_on();
                end
              //else must wait until kbd_getch() reads and turns kbd_cts_on()              
              end
            return;
            break;
          default:               
            kbd_current_byte = kbd_current_byte >> 1;
            kbd_current_byte+=(PIND.3<<7);
            kbd_parity++;
            return;
            break;                   
          end//switch(kbd_edge_count)
  end//if kbd_dir = RX
else //kbd_dir == TX
  begin
  //In order for the MCU to go into TX mode, KBD_CLK must be pulled low for 100us
  //then the KBD_DATA line is pulled low, this is done by kbd_send_cmd and
  //kbd_handler in the ACK state
  kbd_edge_count++;                         
  data = kbd_data_pin;//need to read stop and ack bits
  switch (kbd_edge_count)
          begin         
          case 1://falling edge, start bit 
            kbd_data_port = 0;
            kbd_parity = 1;//if the number of 1's in data even, set parity to 1
            //kbd_cmd_temp = kbd_cmd_byte;//don't modify original cmd
            return;
            break;                    
          case 10://falling edge, send parity bit
	    kbd_data_port = kbd_parity&0x01;
	    return;
            break;
          case 11://falling edge, Change direction of data to: keyboard -> Host
	    //kbd will send stop bit
	    kbd_data_port = 1;//pull up on
	    kbd_data_dir = 0;//input from keyboard
	    //on rising edge of kbd_clk, a 1 will be sent on kbd_data
	    //skipping the rising edge to wait for the ack bit seems to work fine
            return;
            break;
          case 12://Falling edge, expect 0 on kbd_data, ack
            if (data==1)
              begin//errr
              #ifdef debug         
              kbd_cts_off();                                  
              printf("kbd_tx_err, pin = %d\n\r",kbd_data_pin);
              kbd_cts_on();
              #endif     
              kbd_err++;
              if (kbd_err<3) 
                begin
                kbd_send_byte();
                return;
                end
              else 
                begin
                kbd_err=0;
                kbd_send_cmd(0xff,0x00);//reset
		return;
                end
              return;  
              end   
            kbd_dir = RX;
            kbd_edge_count = 0;
            return;
            break;
          default:                                   
            //rising edge from 2 to 9
            //send bit 0 to bit 7 in that order
            kbd_data_port = kbd_cmd_byte & 0x01;      
            if (kbd_cmd_byte&0x01) kbd_parity++;
            kbd_cmd_byte = kbd_cmd_byte >> 1;
            return;
            break;
          end//switch        
  end//kbd_dir == TX 
end                                                                  

#pragma savereg-
//Keyboard Clear To Send signal control
//run kbd_cts_off If we do not want the keyboard to send data
//run kbd_cts_on If we are ready to accept data
void kbd_cts_off(void)
begin
#asm                          
;Turn EXT_INT0 off
push r16
in r16, SREG
push r16
in r16, 0x3b; GIMSK
andi r16, 0xbf; GIMSK = GIMSK | (1<<6) turn EXT_INT0 off
out 0x3b, r16      

SBI 0x11, 2; DDRD.2 = 1, set as output
CBI 0x12, 2; PORTD.2 = 0, pulled low, prevents kbd from sending data

;Clear INT0 flag by writing 1 to INT0
ldi r16, 0x40; INT0 is bit 6 of GIFR
out 0x3a, r16; GIFR

pop r16
out SREG, R16
pop R16
#endasm   
end             

void kbd_cts_on(void)
begin
#asm
CBI 0x11, 2; DDRD.2 = 0, set as input
SBI 0x12, 2; PORTD.2 = 1, clear to send
   
;Clear EXT_INT0 flag, Turn EXT_INT0 on
push r16
in r16, SREG  
push r16        
      
;Turn EXT_INT0 on
in r16, 0x3b; GIMSK
ori r16, 0x40; GIMSK = GIMSK | (1<<6) turn EXT_INT0 on
out 0x3b, r16      
pop r16
out SREG, r16
pop r16
#endasm
end
#pragma savereg+