//keyboard.h
#ifndef begin
#define begin {
#define end }
#endif     

//Constants

//State Machine for kbd ISR
#define WAIT 		0
#define EXT_E0		1  //Extended scancode
#define RELEASE		2
#define GET_CODE	3              
#define EXT_PAUSE	4  //Pause key depressed
#define ECHO		7  //echo
#define ACK		10 //ack and continue sending
#define ACK_END		11 //ack end tx
char eeprom pause_seq[] = {
0xe1,0x14,0x17,0xe1,0xf0,0x14,0xf0,0x77};//Scancode seq for pause
unsigned char p_ptr = 0; //Points to the next code in pause_seq
unsigned char kbd_state; //Statemachine for keeping track of scancodes
bit ext_key = 0; //flag for keeping track of extended key presses
bit pause_key_error = 0;

//Flags
#define SHIFT  1 //Shift-key pressed
#define NOSHIFT 0 //Shift-key not pressed

#define NUMLOCK 1 //must not change this, kbd_numlock true, means numlock is on
#define NONUMLOCK 0
              
#define CAPSLOCK 1
#define NOCAPSLOCK 0

                
//variables           
	
//if hpt == tptr then buffer is full.
//if tptr == (hptr + 1)%kbd_buf_size
//then buffer is empty

unsigned char kbd_current_byte;  
unsigned char kbd_byte_valid;    
unsigned char kbd_err;
unsigned char kbd_ack_err=0;//# of times ack not recieved    
unsigned char kbd_resend_tries=0;
unsigned char kbd_parity;    
unsigned char kbd_cmd_byte;//msg to be sent to kbd
unsigned char kbd_cmd_sent;//last sent byte
unsigned char kbd_cmd_buf[2];//buffer for storing cmd and arguments
unsigned char kbd_dir;    	//0 if MCU in rx mode, 1 in TX mode
#define RX 0 //Keyboard sending data
#define TX 1 //Host to keyboard transmission
               
//functions

void kbd_cts_off(void); 
void kbd_cts_on(void);
void kbd_send_byte(void); 
           

//Macros
#define kbd_ISR_on_falling_edge MCUCR = MCUCR&0xfe;
#define kbd_ISR_on_rising_edge  MCUCR = MCUCR|0x01;
             
#define kbd_int_on GIMSK = GIMSK | (0x01<<6);
#define kbd_int_off GIMSK = GIMSK & ~(0x01<<6); 

#ifndef kbd_clk_port 
#define kbd_clk_port  PORTD.2
#define kbd_data_port PORTD.3
#define kbd_clk_pin   PIND.2
#define kbd_data_pin  PIND.3
#define kbd_clk_dir   DDRD.2
#define kbd_data_dir  DDRD.3
#endif
 
#ifndef BASE_ADDR                      
//Define values for special keys
//These values are special characters in the ASCII table
//since we will not use those characters, we can redefine the table.
#define BASE_ADDR	200
#define LEFT		1 + BASE_ADDR
#define RIGHT           2 + BASE_ADDR
#define UP              3 + BASE_ADDR
#define DOWN            4 + BASE_ADDR
#define BACKSPACE       0x08		//ASCII default
#define INS		6 + BASE_ADDR
#define DEL		7 + BASE_ADDR
#define HOME		8 + BASE_ADDR
#define END		9 + BASE_ADDR                               
#define F1		10 + BASE_ADDR
#define F2		11 + BASE_ADDR
#define F3		12 + BASE_ADDR
#endif

//Look up table to convert from scancode to ascii
//Table is adapted from http://www.beyondlogic.org/keyboard/keybrd.htm
//Thanks to Craig Peacock for the original table for a 68HC11.
unsigned char eeprom table_noshift_nonumlock[] = { 
	      0x00     ,    // 00
              0x00     ,    // 01  F9
              0x00     ,    // 02
              0x00     ,    // 03  F5
              F3       ,    // 04  F3
              F1       ,    // 05  F1
              F2       ,    // 06  F2
              0x00     ,    // 07  F12
              0x00     ,    // 08
              0x00     ,    // 09  F10
              0x00     ,    // 0A  F8
              0x00     ,    // 0B  F6
              0x00     ,    // 0C  F4
              0x09     ,    // 0D  TAB
              '`'      ,    // 0E  ` or ~
              0x00     ,    // 0F

              0x00     ,    // 10
              0x00     ,    // 11  Left ALT
              0x00     ,    // 12  Left SHIFT
              0x00     ,    // 13
              0x00     ,    // 14  Left Ctrl
              'q'      ,    // 15  Q
              '1'      ,    // 16  1 or !
              0x00     ,    // 17
              0x00     ,    // 18
              0x00     ,    // 19
              'z'      ,    // 1A  Z
              's'      ,    // 1B  S
              'a'      ,    // 1C  A
              'w'      ,    // 1D  W
              '2'      ,    // 1E  2 or @
              0x00     ,    // 1F

              0x00     ,    // 20
              'c'      ,    // 21  C
              'x'      ,    // 22  X
              'd'      ,    // 23  D
              'e'      ,    // 24  E
              '4'      ,    // 25  4 or $
              '3'      ,    // 26  3 or #
              0x00     ,    // 27
              0x00     ,    // 28
              ' '      ,    // 29  Space
              'v'      ,    // 2A  V
              'f'      ,    // 2B  F
              't'      ,    // 2C  T
              'r'      ,    // 2D  R
              '5'      ,    // 2E  5 or %
              0x00     ,    // 2F

              0x00     ,    // 30
              'n'      ,    // 31  N
              'b'      ,    // 32  B
              'h'      ,    // 33  H
              'g'      ,    // 34  G
              'y'      ,    // 35  Y
              '6'      ,    // 36  6 or ^
              0x00     ,    // 37
              0x00     ,    // 38
              0x00     ,    // 39
              'm'      ,    // 3A  M
              'j'      ,    // 3B  J
              'u'      ,    // 3C  U
              '7'      ,    // 3D  7 or &
              '8'      ,    // 3E  8 or *
              0x00     ,    // 3F

              0x00     ,    // 40
              ','      ,    // 41  , or <
              'k'      ,    // 42  K
              'i'      ,    // 43  I
              'o'      ,    // 44  O
              '0'      ,    // 45  0 or )
              '9'      ,    // 46  9 or (
              0x00     ,    // 47
              0x00     ,    // 48
              '.'      ,    // 49  . or >
              '/'      ,    // 4A  / or ?
              'l'      ,    // 4B  L
              ';'      ,    // 4C  ; or :
              'p'      ,    // 4D  P
              '-'      ,    // 4E  - or _
              0x00     ,    // 4F

              0x00     ,    // 50
              0x00     ,    // 51
              0x27     ,    // 52  '  or "
              0x00     ,    // 53
              '['      ,    // 54  [ or {
              '='      ,    // 55  = OR +
              0x00     ,    // 56
              0x00     ,    // 57
              0x00     ,    // 58  Caps Lock
              0x00     ,    // 59  Right Shift
              0x0D     ,    // 5A  Enter
              ']'      ,    // 5B  ] or }
              0x00     ,    // 5C
              '\\'     ,    // 5D  \ or |
              0x00     ,    // 5E
              0x00     ,    // 5F

              0x00     ,    // 60
              0x00     ,    // 61
              0x00     ,    // 62
              0x00     ,    // 63
              0x00     ,    // 64
              0x00     ,    // 65
              0x08     ,    // 66  Backspace
              0x00     ,    // 67
              0x00     ,    // 68
              END      ,    // 69  NUM - 1 or END
              0x00     ,    // 6A
              LEFT     ,    // 6B  NUM - 4 or LEFT
              HOME     ,    // 6C  NUM - 7 or HOME
              0x00     ,    // 6D
              0x00     ,    // 6E
              0x00     ,    // 6F

              INS      ,    // 70  NUM - 0 or INS
              DEL      ,    // 71  NUM - . or DEL
              DOWN     ,    // 72  NUM - 2 or DOWN
              0x00     ,    // 73  NUM - 5
              RIGHT    ,    // 74  NUM - 6 or RIGHT
              UP       ,    // 75  NUM - 8 or UP
              0x1B     ,    // 76  ESC
              0x00     ,    // 77  NUM LOCK
              0x00     ,    // 78  F11
              '+'      ,    // 79  NUM - + (Plus)
              0x00     ,    // 7A  NUM 3 or PAGE DOWN
              '-'      ,    // 7B  NUM - - (Minus)
              '*'      ,    // 7C  NUM - *
              0x00     ,    // 7D  NUM - 9 or PAGE UP
              0x00     ,    // 7E  SCROLL LOCK
              0x00     ,    // 7F

              0x00     ,    // 80
              0x00     ,    // 81
              0x00     ,    // 82
              0x00     ,    // 83  F7
              0x00     ,    // 84
              0x00     ,    // 85
              0x00     ,    // 86
              0x00     ,    // 87
              0x00     ,    // 88
              0x00     ,    // 89
              0x00     ,    // 8A
              0x00     ,    // 8B
              0x00     ,    // 8C
              0x00     ,    // 8D
              0x00     ,    // 8E
              0x00     };   // 8F

unsigned char eeprom table_shift_numlock[]={
	      0x00     ,    // 00
              0x00     ,    // 01  F9
              0x00     ,    // 02
              0x00     ,    // 03  F5
              F3       ,    // 04  F3
              F1       ,    // 05  F1
              F2       ,    // 06  F2
              0x00     ,    // 07  F12
              0x00     ,    // 08
              0x00     ,    // 09  F10
              0x00     ,    // 0A  F8
              0x00     ,    // 0B  F6
              0x00     ,    // 0C  F4
              0x09     ,    // 0D  TAB
              '~'      ,    // 0E  ` or ~
              0x00     ,    // 0F

              0x00     ,    // 10
              0x00     ,    // 11  Left ALT
              0x00     ,    // 12  Left SHIFT
              0x00     ,    // 13
              0x00     ,    // 14  Left Ctrl
              'Q'      ,    // 15  Q
              '!'      ,    // 16  1 or !
              0x00     ,    // 17
              0x00     ,    // 18
              0x00     ,    // 19
              'Z'      ,    // 1A  Z
              'S'      ,    // 1B  S
              'A'      ,    // 1C  A
              'W'      ,    // 1D  W
              '@'      ,    // 1E  2 or @
              0x00     ,    // 1F

              0x00     ,    // 20
              'C'      ,    // 21  C
              'X'      ,    // 22  X
              'D'      ,    // 23  D
              'E'      ,    // 24  E
              '$'      ,    // 25  4 or $
              '#'      ,    // 26  3 or #
              0x00     ,    // 27
              0x00     ,    // 28
              ' '      ,    // 29  Space
              'V'      ,    // 2A  V
              'F'      ,    // 2B  F
              'T'      ,    // 2C  T
              'R'      ,    // 2D  R
              '%'      ,    // 2E  5 or %
              0x00     ,    // 2F

              0x00     ,    // 30
              'N'      ,    // 31  N
              'B'      ,    // 32  B
              'H'      ,    // 33  H
              'G'      ,    // 34  G
              'Y'      ,    // 35  Y
              '^'      ,    // 36  6 or ^
              0x00     ,    // 37
              0x00     ,    // 38
              0x00     ,    // 39
              'M'      ,    // 3A  M
              'J'      ,    // 3B  J
              'U'      ,    // 3C  U
              '&'      ,    // 3D  7 or &
              '*'      ,    // 3E  8 or *
              0x00     ,    // 3F

              0x00     ,    // 40
              '<'      ,    // 41  , or <
              'K'      ,    // 42  K
              'I'      ,    // 43  I
              'O'      ,    // 44  O
              ')'      ,    // 45  0 or )
              '('      ,    // 46  9 or (
              0x00     ,    // 47
              0x00     ,    // 48
              '>'      ,    // 49  > or .
              '?'      ,    // 4A  / or ?
              'L'      ,    // 4B  L
              ':'      ,    // 4C  ; or :
              'P'      ,    // 4D  P
              '_'      ,    // 4E  - or _
              0x00     ,    // 4F

              0x00     ,    // 50
              0x00     ,    // 51
              0x22     ,    // 52  '  or "
              0x00     ,    // 53
              '{'      ,    // 54  [ or {
              '+'      ,    // 55  = OR +
              0x00     ,    // 56
              0x00     ,    // 57
              0x00     ,    // 58  Caps Lock
              0x00     ,    // 59  Right Shift
              0x0D     ,    // 5A  Enter
              '}'      ,    // 5B  ] or }
              0x00     ,    // 5C
              '|'      ,    // 5D  \ or |
              0x00     ,    // 5E
              0x00     ,    // 5F

              0x00     ,    // 60
              0x00     ,    // 61
              0x00     ,    // 62
              0x00     ,    // 63
              0x00     ,    // 64
              0x00     ,    // 65
              0x08     ,    // 66  Backspace
              0x00     ,    // 67
              0x00     ,    // 68
              '1'      ,    // 69  NUM - 1 or END
              0x00     ,    // 6A
              '4'      ,    // 6B  NUM - 4 or LEFT
              '7'      ,    // 6C  NUM - 7 or HOME
              0x00     ,    // 6D
              0x00     ,    // 6E
              0x00     ,    // 6F

              '0'      ,    // 70  NUM - 0 or INS
              '.'      ,    // 71  NUM - . or DEL
              '2'      ,    // 72  NUM - 2 or DOWN
              '5'      ,    // 73  NUM - 5
              '6'      ,    // 74  NUM - 6 or RIGHT
              '8'      ,    // 75  NUM - 8 or UP
              0x1B     ,    // 76  ESC
              0x00     ,    // 77  NUM LOCK
              0x00     ,    // 78  F11
              '+'      ,    // 79  NUM - + (Plus)
              '3'      ,    // 7A  NUM 3 or PAGE DOWN
              '-'      ,    // 7B  NUM - - (Minus)
              '*'      ,    // 7C  NUM - *
              '9'      ,    // 7D  NUM - 9 or PAGE UP
              0x00     ,    // 7E  SCROLL LOCK
              0x00     ,    // 7F

              0x00     ,    // 80
              0x00     ,    // 81
              0x00     ,    // 82
              0x00     ,    // 83  F7
              0x00     ,    // 84
              0x00     ,    // 85
              0x00     ,    // 86
              0x00     ,    // 87
              0x00     ,    // 88
              0x00     ,    // 89
              0x00     ,    // 8A
              0x00     ,    // 8B
              0x00     ,    // 8C
              0x00     ,    // 8D
              0x00     ,    // 8E
              0x00     };   // 8F

unsigned char flash table_shift_nonumlock[] = {  

	      0x00     ,    // 00
              0x00     ,    // 01  F9
              0x00     ,    // 02
              0x00     ,    // 03  F5
              F3       ,    // 04  F3
              F1       ,    // 05  F1
              F2       ,    // 06  F2
              0x00     ,    // 07  F12
              0x00     ,    // 08
              0x00     ,    // 09  F10
              0x00     ,    // 0A  F8
              0x00     ,    // 0B  F6
              0x00     ,    // 0C  F4
              0x09     ,    // 0D  TAB
              '~'      ,    // 0E  ` or ~
              0x00     ,    // 0F

              0x00     ,    // 10
              0x00     ,    // 11  Left ALT
              0x00     ,    // 12  Left SHIFT
              0x00     ,    // 13
              0x00     ,    // 14  Left Ctrl
              'Q'      ,    // 15  Q
              '!'      ,    // 16  1 or !
              0x00     ,    // 17
              0x00     ,    // 18
              0x00     ,    // 19
              'Z'      ,    // 1A  Z
              'S'      ,    // 1B  S
              'A'      ,    // 1C  A
              'W'      ,    // 1D  W
              '@'      ,    // 1E  2 or @
              0x00     ,    // 1F

              0x00     ,    // 20
              'C'      ,    // 21  C
              'X'      ,    // 22  X
              'D'      ,    // 23  D
              'E'      ,    // 24  E
              '$'      ,    // 25  4 or $
              '#'      ,    // 26  3 or #
              0x00     ,    // 27
              0x00     ,    // 28
              ' '      ,    // 29  Space
              'V'      ,    // 2A  V
              'F'      ,    // 2B  F
              'T'      ,    // 2C  T
              'R'      ,    // 2D  R
              '%'      ,    // 2E  5 or %
              0x00     ,    // 2F

              0x00     ,    // 30
              'N'      ,    // 31  N
              'B'      ,    // 32  B
              'H'      ,    // 33  H
              'G'      ,    // 34  G
              'Y'      ,    // 35  Y
              '^'      ,    // 36  6 or ^
              0x00     ,    // 37
              0x00     ,    // 38
              0x00     ,    // 39
              'M'      ,    // 3A  M
              'J'      ,    // 3B  J
              'U'      ,    // 3C  U
              '&'      ,    // 3D  7 or &
              '*'      ,    // 3E  8 or *
              0x00     ,    // 3F

              0x00     ,    // 40
              '<'      ,    // 41  , or <
              'K'      ,    // 42  K
              'I'      ,    // 43  I
              'O'      ,    // 44  O
              ')'      ,    // 45  0 or )
              '('      ,    // 46  9 or (
              0x00     ,    // 47
              0x00     ,    // 48
              '>'      ,    // 49  > or .
              '?'      ,    // 4A  / or ?
              'L'      ,    // 4B  L
              ':'      ,    // 4C  ; or :
              'P'      ,    // 4D  P
              '_'      ,    // 4E  - or _
              0x00     ,    // 4F

              0x00     ,    // 50
              0x00     ,    // 51
              0x22     ,    // 52  '  or "
              0x00     ,    // 53
              '{'      ,    // 54  [ or {
              '+'      ,    // 55  = OR +
              0x00     ,    // 56
              0x00     ,    // 57
              0x00     ,    // 58  Caps Lock
              0x00     ,    // 59  Right Shift
              0x0D     ,    // 5A  Enter
              '}'      ,    // 5B  ] or }
              0x00     ,    // 5C
              '|'      ,    // 5D  \ or |
              0x00     ,    // 5E
              0x00     ,    // 5F

	      0x00     ,    // 60
              0x00     ,    // 61
              0x00     ,    // 62
              0x00     ,    // 63
              0x00     ,    // 64
              0x00     ,    // 65
              0x08     ,    // 66  Backspace
              0x00     ,    // 67
              0x00     ,    // 68
              END      ,    // 69  NUM - 1 or END
              0x00     ,    // 6A
              LEFT     ,    // 6B  NUM - 4 or LEFT
              HOME     ,    // 6C  NUM - 7 or HOME
              0x00     ,    // 6D
              0x00     ,    // 6E
              0x00     ,    // 6F

              INS      ,    // 70  NUM - 0 or INS
              DEL      ,    // 71  NUM - . or DEL
              DOWN     ,    // 72  NUM - 2 or DOWN
              0x00     ,    // 73  NUM - 5
              RIGHT    ,    // 74  NUM - 6 or RIGHT
              UP       ,    // 75  NUM - 8 or UP
              0x1B     ,    // 76  ESC
              0x00     ,    // 77  NUM LOCK
              0x00     ,    // 78  F11
              '+'      ,    // 79  NUM - + (Plus)
              0x00     ,    // 7A  NUM 3 or PAGE DOWN
              '-'      ,    // 7B  NUM - - (Minus)
              '*'      ,    // 7C  NUM - *
              0x00     ,    // 7D  NUM - 9 or PAGE UP
              0x00     ,    // 7E  SCROLL LOCK
              0x00     ,    // 7F
                            
              0x00     ,    // 80
              0x00     ,    // 81
              0x00     ,    // 82
              0x00     ,    // 83  F7
              0x00     ,    // 84
              0x00     ,    // 85
              0x00     ,    // 86
              0x00     ,    // 87
              0x00     ,    // 88
              0x00     ,    // 89
              0x00     ,    // 8A
              0x00     ,    // 8B
              0x00     ,    // 8C
              0x00     ,    // 8D
              0x00     ,    // 8E
              0x00     };   // 8F

unsigned char flash table_noshift_numlock[] = {
	      0x00     ,    // 00
              0x00     ,    // 01  F9
              0x00     ,    // 02
              0x00     ,    // 03  F5
              F3       ,    // 04  F3
              F1       ,    // 05  F1
              F2       ,    // 06  F2
              0x00     ,    // 07  F12
              0x00     ,    // 08
              0x00     ,    // 09  F10
              0x00     ,    // 0A  F8
              0x00     ,    // 0B  F6
              0x00     ,    // 0C  F4
              0x09     ,    // 0D  TAB
              '`'      ,    // 0E  ` or ~
              0x00     ,    // 0F

              0x00     ,    // 10
              0x00     ,    // 11  Left ALT
              0x00     ,    // 12  Left SHIFT
              0x00     ,    // 13
              0x00     ,    // 14  Left Ctrl
              'q'      ,    // 15  Q
              '1'      ,    // 16  1 or !
              0x00     ,    // 17
              0x00     ,    // 18
              0x00     ,    // 19
              'z'      ,    // 1A  Z
              's'      ,    // 1B  S
              'a'      ,    // 1C  A
              'w'      ,    // 1D  W
              '2'      ,    // 1E  2 or @
              0x00     ,    // 1F

              0x00     ,    // 20
              'c'      ,    // 21  C
              'x'      ,    // 22  X
              'd'      ,    // 23  D
              'e'      ,    // 24  E
              '4'      ,    // 25  4 or $
              '3'      ,    // 26  3 or #
              0x00     ,    // 27
              0x00     ,    // 28
              ' '      ,    // 29  Space
              'v'      ,    // 2A  V
              'f'      ,    // 2B  F
              't'      ,    // 2C  T
              'r'      ,    // 2D  R
              '5'      ,    // 2E  5 or %
              0x00     ,    // 2F

              0x00     ,    // 30
              'n'      ,    // 31  N
              'b'      ,    // 32  B
              'h'      ,    // 33  H
              'g'      ,    // 34  G
              'y'      ,    // 35  Y
              '6'      ,    // 36  6 or ^
              0x00     ,    // 37
              0x00     ,    // 38
              0x00     ,    // 39
              'm'      ,    // 3A  M
              'j'      ,    // 3B  J
              'u'      ,    // 3C  U
              '7'      ,    // 3D  7 or &
              '8'      ,    // 3E  8 or *
              0x00     ,    // 3F

              0x00     ,    // 40
              ','      ,    // 41  , or <
              'k'      ,    // 42  K
              'i'      ,    // 43  I
              'o'      ,    // 44  O
              '0'      ,    // 45  0 or )
              '9'      ,    // 46  9 or (
              0x00     ,    // 47
              0x00     ,    // 48
              '.'      ,    // 49  . or >
              '/'      ,    // 4A  / or ?
              'l'      ,    // 4B  L
              ';'      ,    // 4C  ; or :
              'p'      ,    // 4D  P
              '-'      ,    // 4E  - or _
              0x00     ,    // 4F

              0x00     ,    // 50
              0x00     ,    // 51
              0x27     ,    // 52  '  or "
              0x00     ,    // 53
              '['      ,    // 54  [ or {
              '='      ,    // 55  = OR +
              0x00     ,    // 56
              0x00     ,    // 57
              0x00     ,    // 58  Caps Lock
              0x00     ,    // 59  Right Shift
              0x0D     ,    // 5A  Enter
              ']'      ,    // 5B  ] or }
              0x00     ,    // 5C
              '\\'     ,    // 5D  \ or |
              0x00     ,    // 5E
              0x00     ,    // 5F
              0x00     ,    // 60
              0x00     ,    // 61
              0x00     ,    // 62
              0x00     ,    // 63
              0x00     ,    // 64
              0x00     ,    // 65
              0x08     ,    // 66  Backspace
              0x00     ,    // 67
              0x00     ,    // 68
              '1'      ,    // 69  NUM - 1 or END
              0x00     ,    // 6A
              '4'      ,    // 6B  NUM - 4 or LEFT
              '7'      ,    // 6C  NUM - 7 or HOME
              0x00     ,    // 6D
              0x00     ,    // 6E
              0x00     ,    // 6F

              '0'      ,    // 70  NUM - 0 or INS
              '.'      ,    // 71  NUM - . or DEL
              '2'      ,    // 72  NUM - 2 or DOWN
              '5'      ,    // 73  NUM - 5
              '6'      ,    // 74  NUM - 6 or RIGHT
              '8'      ,    // 75  NUM - 8 or UP
              0x1B     ,    // 76  ESC
              0x00     ,    // 77  NUM LOCK
              0x00     ,    // 78  F11
              '+'      ,    // 79  NUM - + (Plus)
              '3'      ,    // 7A  NUM 3 or PAGE DOWN
              '-'      ,    // 7B  NUM - - (Minus)
              '*'      ,    // 7C  NUM - *
              '9'      ,    // 7D  NUM - 9 or PAGE UP
              0x00     ,    // 7E  SCROLL LOCK
              0x00     ,    // 7F

              0x00     ,    // 80
              0x00     ,    // 81
              0x00     ,    // 82
              0x00     ,    // 83  F7
              0x00     ,    // 84
              0x00     ,    // 85
              0x00     ,    // 86
              0x00     ,    // 87
              0x00     ,    // 88
              0x00     ,    // 89
              0x00     ,    // 8A
              0x00     ,    // 8B
              0x00     ,    // 8C
              0x00     ,    // 8D
              0x00     ,    // 8E
              0x00     };   // 8F         
