//main.c keyboard
#include <Mega163.h>
#include <stdio.h>  //sprintf
#include <stdlib.h>
#include <delay.h>
#include <ctype.h>       

//#define debug 1         
#define allow_eeprom_write 1 //enable this only in final version, conserve write-erase cycles

#include "global.h"//Global function prototypes and variables
#include "UI.h"//user interface function prototypes and variables

#ifndef begin
#define begin {
#define end }
#endif

void initialize(void)
begin
UBRR = 51; //9600 BAUD     
UCSRB = 0x10 + 0x08;
UCSRB = (1<<4)|(1<<3);//RXEN,TXZN

//printf("init sensors...\n\r");
//delay_ms(10);//so that printf not interrupted by ADC
initialize_sensors();
//printf("end init sensors\n\r");

//init keyboard
initialize_keyboard();

//init timers
TCCR1B = 0x08 + 0x04;//Clear on OC1A match and prescale CK/256
OCR1A = 31249U; //31250 Cycles
TIMSK = 0x10;//Timer1 OCR1A match interrupt enable

//init clock
seconds = 0;
minutes = 0;
hours	= 0;

#asm   
  sei ;//Activate interrupts
#endasm
         
delay_ms(50);//wait for STV5730A
//init STV
//printf("init STV...\n\r");
STV_initialize();
//printf("End init STV\n\r");
                    
//Show menu
UI_initialize();
end

 

void main(void)
begin       
#ifdef debug
#endif
initialize();                     
//printf("kbd_buf_size %x, KBS = %x,uc KBS=%x ",kbd_buf_size,KBD_BUF_SIZE,(unsigned char) KBD_BUF_SIZE);
printf("Entering while loop\n\r");
//printf("UI_mon = %u\n\r",UI_monitor_time);  
while(1)
    begin                                   
    //printf("UI_mon = %u\n\r",UI_monitor_time);  
    if (UI_monitor_time==0)
      begin    
      //printf(" %d ",kbd_edge_count);                               
      UI_monitor();
      UI_show_time();
      UI_monitor_time = UI_MONITOR_RELOAD;
      end                                 
    if (STV_monitor_time==0)
      begin        
      STV_monitor();
      STV_monitor_time = STV_MONITOR_RELOAD;
      end
    if (UI_show_sensor_time==0)
      begin                     
      UI_show_sensors();
      UI_show_sensor_time = UI_SHOW_SENSOR_RELOAD;
      #ifdef debug
      UI_debug();
      #endif
      
      end
    end//while
 end                              
                  
//Update the date by moving forward 1 day      
void inc_day(void)                       
begin
day++;
switch(month)
  begin
  //febuary
  case 2:
    //check that year is divisible by 4(leap year)
    //since we only record last 2 digits of the year
    //this will be accurate only until 2099, 2100 is not a leap year
    if ((year&0x03)==0)
      begin
      //year is divisible by 4, hence can safely assume
      //that it is a leap year
      if (day==30) 
        begin
        day=1;                        
        month++;
        end
      end
     else
       begin
       //not a leap year
       if (day==29)
         begin
         day=1;
         month++;
         end
       end               
    break;
  //months with 31 days
  case 1:
  case 3:
  case 5:
  case 7:
  case 8:
  case 10:
  case 12:
    if (day==32) 
      begin
      day = 1;
      month++;
      end
    break;
  default:
    if (day==31) 
      begin
      day=1;
      month++;
      end
    break;
  end //switch(month)    
if (month==13) 
  begin
  month = 1;
  year++;
  end
end
       
//User interface
void UI_initialize(void)
begin                      
STV_gotoxy(0,1);
STV_printf("WeatherStat by Zhihern Loh",WHITE,0,0);
STV_gotoxy(0,2);
STV_printf("Version 1.00",WHITE,0,0);
STV_gotoxy(0,3);
STV_printf("www.lohzh.com",WHITE,0,0);
STV_gotoxy(0,4);
STV_printf("Press F1 for help",WHITE,1,0);
STV_gotoxy(0,9);
STV_printf("Date format is MM/DD/YY",WHITE,0,0);
delay_ms(2000U);
STV_clrscr(); 
UI_clr_text_buf();
//Start timer0, which sets the time for UI_monitor
UI_monitor_time = UI_MONITOR_RELOAD;
STV_monitor_time = STV_MONITOR_RELOAD;
UI_show_sensor_time = UI_SHOW_SENSOR_RELOAD;
TCCR0 = TIMER0_PRESCALAR;
TIMSK = TIMSK | 0x01;//Timer0 ovrfl int enable
end  


void UI_show_sensors(void)
begin                         
unsigned char str[28],tstr1[10],tstr2[10],adc0[8],adc1[8];
get_ADC(adc0,ADC_SENSOR_CHANNEL0);
get_ADC(adc1,ADC_SENSOR_CHANNEL1);
get_temperature(tstr1);
get_pressure(tstr2);                                   

STV_gotoxy(0,0);
STV_printf(clrline,WHITE,0,0);
STV_gotoxy(0,0);
sprintf(str,"Temp %s Pres %s",tstr1,tstr2);
STV_print(str,WHITE,0,0);         

STV_gotoxy(0,10);
STV_printf(clrline,WHITE,0,0);
STV_gotoxy(0,10);
sprintf(str,"ADC0: %s ADC1: %s",adc0,adc1);
STV_print(str,WHITE,0,0);
end

void UI_show_time(void)
begin
unsigned char time[28];
sprintf(time,"%02d:%02d:%02d %02d/%02d/%02d",hours,minutes,seconds,month,day,year);
STV_gotoxy(0,9);
STV_print(time,WHITE,0,0);
end                                    
                  
void UI_clr_text_buf(void)
begin
unsigned char x,y;
for (x=0;x<28;x++)
  for (y=0;y<10;y++)
    text_display[x,y] = ' ';
text_x = 0;
text_y = 0;
end        

void UI_show_text(void)
begin
unsigned char x,y;
/*          
//text_x, text_y points to element after last
if (text_y>0)//write the full length lines, then write the last line
  for (y=0;y<text_y;y++)
    for (x=0;x<TEXT_WIDTH;x++)
       begin
       STV_gotoxy(x + TEXT_X_OFFSET, y + TEXT_Y_OFFSET);
       STV_putch(text_display[x,y],WHITE,0,0);
       end
//write the last line
if (text_x>0)
  for (x=0;x<text_x;x++)
    begin
    STV_gotoxy(x + TEXT_X_OFFSET, text_y + TEXT_Y_OFFSET);
    STV_putch(text_display[x,text_y],WHITE,0,0);
    end*/           
for (y=0;y<TEXT_HEIGHT;y++)
  begin
  STV_gotoxy(TEXT_X_OFFSET,y+TEXT_Y_OFFSET);
  for (x=0;x<TEXT_WIDTH;x++)
    STV_putch(text_display[x,y],WHITE,0,0);
  end//for text_y
end

void UI_hide_text(void)
begin
unsigned char y;
for (y=0;y<TEXT_HEIGHT;y++)
  begin
  STV_gotoxy(0,y+TEXT_Y_OFFSET);
  STV_printf(clrline,WHITE,0,0);
  end
end

void UI_debug(void)
begin       
DDRC = 0;//input
PORTC = 1;//pull up on              

if (PINC.0==0) //show kbd status
  begin
  printf("kbd_hptr=%d, tptr = %d\n\r",kbd_hptr,kbd_tptr);
  printf("kbd_buf[hptr]=%c,kbd_buf[tptr]=%c",kbd_buf[kbd_hptr],kbd_buf[kbd_tptr]);
  end   

end

//UI_monitor - monitors user keypresses
void UI_monitor(void)
begin           
unsigned char c, tstr[29],aref_str[29],mpx_str[29];//temp variables
//get commands
c = kbd_getch();
if (c!=0) //if a key has been pressed...
  switch(UI_monitor_status)
    begin
    case CMD_MODE:
      switch(c)
              begin
              case 'e':
              case 'E':
                UI_monitor_status = TEXT_ENTRY_MODE;
                UI_show_text();                                         
                //Write cursor
                STV_gotoxy(text_x + TEXT_X_OFFSET, text_y + TEXT_Y_OFFSET);
                STV_putcode(0x63,WHITE,1,0);//blink,no background
                #ifdef debug
                printf("Test kbd entry mode\n\r");
                #endif
                break;
              case 'c':
              case 'C'://clear text buffer
                UI_clr_text_buf(); 
                UI_hide_text();
                STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET+1);
                STV_printf("Buffer cleared. Any key to cont.",WHITE,0,0);
                UI_monitor_status = WAIT_4_KEYPRESS;
                #ifdef debug
                printf("Clear text buffer\n\r");
                #endif
                break;
              case 't':
              case 'T':                       
              	UI_monitor_status = SET_TIME_MODE;
              	UI_hide_text();
              	UI_set_time_status = 0;           
              	tyear=tmonth=tday=thours=tminutes=0;
              	STV_gotoxy(0,3);
              	STV_printf("Enter date and time",WHITE,0,0);
              	STV_gotoxy(0,4);
              	STV_printf("MM/DD/YY",WHITE,0,0);
              	STV_gotoxy(0,5);
              	STV_putcode(0x63,WHITE,1,0);//draw cursor
              	STV_printf("0/00/00",WHITE,0,0);
              	STV_gotoxy(0,6);
              	STV_printf("HH:MM",WHITE,0,0); 
              	STV_gotoxy(0,7);
              	STV_printf("00:00",WHITE,0,0);
              	#ifdef debug
              	printf("Time entry mode\n\r");
              	#endif
                break;
              case 'r':
              case 'R':
                UI_monitor_status = SET_VREF_MODE;
                UI_hide_text();        
                UI_set_vref_status = 0;
                //Show present reference voltages
                ftoa(aref,2,aref_str);
                ftoa(mpx2100_supply_V,2,mpx_str);
                sprintf(tstr,"AREF:%sV MPX Vsupply:%sV",aref_str,mpx_str);
                STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET);
                STV_print(tstr,WHITE,0,0);                                
                STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET+1);
                STV_printf("Enter new reference voltages",WHITE,0,0);
                STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET+2);
                STV_printf("new AREF:0.00",WHITE,0,0);
                STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET+3);
                STV_printf("new MPX Vsupply:0.00",WHITE,0,0);
                STV_gotoxy(9+TEXT_X_OFFSET,TEXT_Y_OFFSET+2);
                STV_putcode(0x63,WHITE,1,0);//draw cursor
                #ifdef debug
                printf("Vref entry mode\n\r");
                #endif
                break;
              case 'd':
              case 'D'://Show reference voltage values
                UI_monitor_status = WAIT_4_KEYPRESS;
                UI_hide_text();
                //Show present reference voltages
                ftoa(aref,2,aref_str);
                ftoa(mpx2100_supply_V,2,mpx_str);
                sprintf(tstr,"AREF:%sV MPX Vsupply:%sV",aref_str,mpx_str);
                STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET);
                STV_print(tstr,WHITE,0,0);                                
                STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET+1);
                STV_printf("Press any key to continue",WHITE,1,0);
                break;
              case F1://show help
                UI_monitor_status = WAIT_4_KEYPRESS;
                UI_hide_text();
                STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET);    
                	  //0123456789012345678901234567<-Max no. of char
                STV_printf("Help Screen -Any key to cont",WHITE,1,0);//0
	        STV_printf("User manual at www.lohzh.com",WHITE,0,0);//1
                STV_printf("e:edit text    c :clear text",WHITE,0,0);//2
                STV_printf("F2:Show text   F3:Hide  text",WHITE,0,0);//3
                STV_printf("v:overlay NTSC f :no NTSC in",WHITE,0,0);//4
                STV_printf("t: set time and date        ",WHITE,0,0);//5
                STV_printf("r: set reference voltages   ",WHITE,0,0);//6
                STV_printf("d: show reference voltages  ",WHITE,0,0);//7
                break;                              
              case F2://Clear displayed text
                UI_show_text();
                #ifdef debug
                printf("Showing text\n\r");
                #endif
                break;                     
              case F3://Show text
                UI_hide_text();
                #ifdef debug
                printf("Hiding text\n\r");
                #endif
                break;
      	      case 'V':
              case 'v':
                printf("video mode\n\r");
        	STV_video_mode();
                break; 
              case 'F':
              case 'f':
                printf("full page mode\n\r");
              	STV_full_page_mode();
                break; 
             /*   
              case 'm':
                printf("send mode register\n\r");
                STV_set_mode_reg();
                break;
              case 'M':
                printf(" set mode reg2\n\r");
                STV_set_mode_reg2();*/
              default:                
                #ifdef debug
                printf(", cmd not recog.\n\r");
                #endif
                break;
              end//switch(c)
      break;
    case TEXT_ENTRY_MODE:
      //write the char to the buffer, check for special keys
      switch(c)
        begin
        case 0x1b://User pressed ESC, terminate text entry
        	UI_monitor_status = CMD_MODE;
	        //Erase the cursor
	        STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
	        STV_putch(text_display[text_x,text_y],WHITE,0,0);
	        break;
	case '\r'://User pressed return, goto next line
		//Erase the cursor
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
	        STV_putch(text_display[text_x,text_y],WHITE,0,0);
	        text_x = 0;
	        text_y++;
	        if (text_y==TEXT_HEIGHT) text_y = 0;
	        //draw the cursor again
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
		STV_putcode(0x63,WHITE,1,0);//cursor character
		UI_monitor_status = TEXT_ENTRY_MODE;		
		break;
	case LEFT:    
		//Erase the cursor
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
	        STV_putch(text_display[text_x,text_y],WHITE,0,0);
	        if (text_x>0) text_x--;
	        else text_x = TEXT_WIDTH-1;
       	        //draw the cursor again
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
		STV_putcode(0x63,WHITE,1,0);//cursor character
		UI_monitor_status = TEXT_ENTRY_MODE;
		break;
	case RIGHT:
		//Erase the cursor
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
	        STV_putch(text_display[text_x,text_y],WHITE,0,0);
		if (text_x==TEXT_WIDTH-1) text_x=0;
		else text_x++;
	        //draw the cursor again
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
		STV_putcode(0x63,WHITE,1,0);//cursor character
		UI_monitor_status = TEXT_ENTRY_MODE;
		break;
	case UP:                                                 
		//Erase the cursor
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
	        STV_putch(text_display[text_x,text_y],WHITE,0,0);
		if (text_y>0) text_y--;
		else text_y = TEXT_HEIGHT;	        
	        //draw the cursor again
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
		STV_putcode(0x63,WHITE,1,0);//cursor character
		UI_monitor_status = TEXT_ENTRY_MODE;
		break;
	case DOWN:                                               
		//Erase the cursor
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
	        STV_putch(text_display[text_x,text_y],WHITE,0,0);
	        if (text_y==TEXT_HEIGHT-1) text_y=0;
	        else text_y++;
	        //draw the cursor again
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
		STV_putcode(0x63,WHITE,1,0);//cursor character
		UI_monitor_status = TEXT_ENTRY_MODE;
		break;
	case BACKSPACE://delete the previous character from the text_display buffer
		//clr the cursor
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
		STV_putch(' ',WHITE,0,0);
		if (text_x>0) 
		  text_x--;
		else //text_x==0
		  if (text_y>0)//if not first character
		    begin
		    text_x = TEXT_WIDTH-1;
		    text_y--;                          
		    text_display[text_x,text_y] = ' ';
                    end
		//draw the cursor again
		STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
		STV_putcode(0x63,WHITE,1,0);//cursor character
		UI_monitor_status = TEXT_ENTRY_MODE;
		break;
	default:
                STV_gotoxy(text_x+TEXT_X_OFFSET,text_y+TEXT_Y_OFFSET);
                if (STV_putch(c,WHITE,0,0)==0) break;//not valid character
                else STV_putcode(0x63,WHITE,1,0);//move cursor
                text_display[text_x,text_y] = c;//store current character
                text_x++;
                if (text_x == TEXT_WIDTH)
                  begin
                  text_x = 0;
                  text_y++;
                  if (text_y==TEXT_HEIGHT) text_y = 0;//loop back to top
                  end 
                UI_monitor_status = TEXT_ENTRY_MODE;
	   	break;                
	end//switch(c)
      break;//End case TEXT_ENTRY_MODE	   	
    case SET_TIME_MODE:
      if (isdigit(c))
              begin
              switch(UI_set_time_status)
                begin
                case 0://get month digit 1
                  tmonth = toint(c) * 10;
                  STV_gotoxy(0,5);
                  STV_putch(c,WHITE,0,0);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor
                  break;
                case 1://get month digit 0                 
                  tmonth += toint(c);
                  STV_gotoxy(1,5);
                  STV_putch(c,WHITE,0,0);
                  STV_putch('/',WHITE,0,0);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor                  
                  break;                                   
                case 2://get day digit 1                   
                  tday = toint(c) * 10;
                  STV_gotoxy(3,5);
                  STV_putch(c,WHITE,0,0);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor
                  break;
                case 3://get day digit 0
                  tday += toint(c);
                  STV_gotoxy(4,5);
                  STV_putch(c,WHITE,0,0);
                  STV_putch('/',WHITE,0,0);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor                  
                  break;
                case 4://get year digit 1
                  tyear = toint(c) * 10;
                  STV_gotoxy(6,5);
                  STV_putch(c,WHITE,0,0);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor
                  break;
                case 5://get year digit 0
                  tyear += toint(c);
                  STV_gotoxy(7,5);
                  STV_putch(c,WHITE,0,0);
                  STV_gotoxy(0,7);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor
                  break;
                case 6://get hour digit 1
                  thours = toint(c) * 10;
                  STV_gotoxy(0,7);
                  STV_putch(c,WHITE,0,0);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor                  
                  break;
                case 7://get hour digit 0                  
                  thours += toint(c);
                  STV_gotoxy(1,7);
                  STV_putch(c,WHITE,0,0);                                    
                  STV_putch(':',WHITE,0,0);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor                  
                  break;
                case 8://get min digit 1                                     
                  tminutes = toint(c) * 10;
                  STV_gotoxy(3,7);
                  STV_putch(c,WHITE,0,0);
                  STV_putcode(0x63,WHITE,1,0);//draw cursor                  
                  break;
                case 9://get min digit 0
                  tminutes += toint(c);
                  seconds = 0;
                  minutes = tminutes;
                  hours = thours;
                  year= tyear; 
                  month = tmonth;
                  day = tday;
                  UI_monitor_status = CMD_MODE;                  
	          UI_hide_text();
                  break;
                default:
                  #ifdef debug
                  printf("SET_TIME_MODE err!\n\r");
                  #endif
                  break;
                end//switch(UI_set_time_status)
              UI_set_time_status++;
              end//isdigit(c)
      break;                           
    case SET_VREF_MODE:                
      if (isdigit(c))
        begin
        switch(UI_set_vref_status)
          begin//aref = x.xx Volts
          case 0://get digit 1 for aref
            new_aref_int = ((unsigned int) toint(c))*100;
            STV_gotoxy(TEXT_X_OFFSET+9,TEXT_Y_OFFSET+2);
            STV_putch(c,WHITE,0,0);
            STV_putch('.',WHITE,0,0);
            STV_putcode(0x63,WHITE,1,0);//draw cursor
            break;
          case 1://get digit 2 for aref
            new_aref_int += ((unsigned int) toint(c)) * 10;
            STV_gotoxy(TEXT_X_OFFSET+11,TEXT_Y_OFFSET+2);
            STV_putch(c,WHITE,0,0);
            STV_putcode(0x63,WHITE,1,0);//draw cursor
            break;
          case 2://get digit 3 for aref                                                        
            new_aref_int += ((unsigned int) toint(c));
            STV_gotoxy(TEXT_X_OFFSET+12,TEXT_Y_OFFSET+2);
            STV_putch(c,WHITE,0,0);
            STV_gotoxy(TEXT_X_OFFSET+16,TEXT_Y_OFFSET+3);
            STV_putcode(0x63,WHITE,1,0);//draw cursor
            break;         
          case 3://get digit 1 for mpx ref
            new_mpx_supply_int = ((unsigned int) toint(c))*100;
            STV_gotoxy(TEXT_X_OFFSET+16,TEXT_Y_OFFSET+3);
            STV_putch(c,WHITE,0,0);
            STV_putch('.',WHITE,0,0);
            STV_putcode(0x63,WHITE,1,0);//draw cursor
            break;                                                 
          case 4:                                      
            new_mpx_supply_int += ((unsigned int) toint(c))*10;       
            STV_gotoxy(TEXT_X_OFFSET+18,TEXT_Y_OFFSET+3);
            STV_putch(c,WHITE,0,0);
            STV_putcode(0x63,WHITE,1,0);//draw cursor
            break;
          case 5:
            UI_monitor_status = WAIT_4_KEYPRESS;
            new_mpx_supply_int += ((unsigned int) toint(c));
            STV_gotoxy(TEXT_X_OFFSET+19,TEXT_Y_OFFSET+3);
            STV_putch(c,WHITE,0,0);
            STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET+4);
            STV_printf("EEPROM will be modified!",WHITE,1,0);
            STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET+5);
            STV_printf("Max 100000 write erase cycles",WHITE,0,0);
            STV_gotoxy(TEXT_X_OFFSET,TEXT_Y_OFFSET+6);
            STV_printf("Press any key to continue",WHITE,1,0);
            new_aref_float = (float) new_aref_int / 100.0;
            new_mpx_supply_float = (float) new_mpx_supply_int / 100.0;
            //Now modify eeprom contents
            #ifdef allow_eeprom_write
            aref = new_aref_float;
            mpx2100_supply_V = new_mpx_supply_float; 
            #endif
            #ifdef debug
            printf("\n\rint aref %u mpx %u\n\r",new_aref_int,new_mpx_supply_int);
            ftoa(new_aref_float,2,aref_str);
            ftoa(new_mpx_supply_float,2,mpx_str);
            printf("new aref = %s, mpx = %s\n\r",aref_str,mpx_str);
            #endif
            break;
            
          end//switch(UI_set_vref_status)
        UI_set_vref_status++;
        end
      break;
    case WAIT_4_KEYPRESS://clr text buf and go back to CMD_MODE
      UI_hide_text();
      UI_monitor_status = CMD_MODE;
      break;      
    default://error in state machine
      STV_clrscr();
      UI_monitor_status = CMD_MODE;
      #ifdef debug
      printf("Err, invalid state in UI_monitor, monitor_status=%d\n\r",UI_monitor_status);
      printf("kbd_hptr=%d,kbd_tptr=%d\n\r",kbd_hptr,kbd_tptr);
      #endif
      break;
    end//switch(UI_monitor_status)
end   

interrupt [TIM1_COMPA] void one_second(void)
begin
seconds++;
if (seconds==60)
  begin
  seconds=0;
  minutes++;
  if (minutes==60)
    begin
    minutes=0;
    hours++;
    if (hours==24) 
      begin
      hours=0;
      inc_day();
      end
    end//minutes
  end
end       
   
//This interrupt occurs every 1.28E-4 s
interrupt [TIM0_OVF] void timer0_ovrfl(void)
begin
if (UI_monitor_time>0) 		UI_monitor_time--;
if (UI_show_sensor_time>0) 	UI_show_sensor_time--;
if (STV_monitor_time>0)		STV_monitor_time--;
end
 