//sensors.c
/*Sensor functions
*/
#include <Mega163.h>
#include <stdlib.h>
#include <string.h>
#include "sensor.h"

#asm
  .equ __w1_port=0x18 	;PORTB 0x15 PORTC
  .equ __w1_bit=2	;pin 0 used for 1 wire bus
#endasm                                    
#include <ds1820.h>

#ifndef begin
#define begin {
#define end }
#endif

//#define DEBUG_SEN 1 //Enable to see printf err msgs

/* maximum number of DS1820/DS18S20 connected to the bus */
#define MAX_DEVICES 8
/* DS1820/DS18S20 devices ROM code storage area,
   9 bytes are used for each device
   (see the w1_search function description),
   but only the first 8 bytes contain the ROM code
   and CRC */
unsigned char rom_codes[MAX_DEVICES,9];

//Run initialize_sensors() after 'sei' - int enabled.
void initialize_sensors(void)
begin
unsigned char devices;//no. of devices on 1-wire bus
ADCSR = ADCSR_DEFAULT;              
//printf("ADCSR %x\n\r",ADCSR); 
//Set ADCSR - ADC control and status register

//Search for ds1820, detect no. of ds18s20 devices
//and store their rom codes in the rom_codes array
//note that an external pull-up is required on the data bus
devices = w1_init();//init 1-wire bus
#ifdef DEBUG_SEN
if (devices) printf("Devices present on 1-wire bus\n\r");
else printf("No devices detected on 1-wire bus\n\r");
devices = w1_search(0xf0,rom_codes);
printf("%d ds18s20 detected\n\r",devices);           
printf("DDRB=%x,PORTB=%x,PINB=%x\n\r",DDRB,PORTB,PINB);
#endif
end                           

float ADC_get(unsigned char channel)       
begin                      
unsigned int ADC_value;                    
float voltage;
unsigned char MCUCR_save;
ADMUX = SELECTED_V_REF + channel;//Set ADMUX, multiplexor ADC
MCUCR_save = MCUCR;//save MCUCR because we're going to disable other interrupts
//Set sleep mode
//SE bit in MCUCR must be set(one)
//Set SM1/SMO = 01 for ADC noise reduction mode
//Execute sleep instruction, MCU will enter
//ADC noise reduction mode, if ADC is enabled,
//conversion starts automaticaly
//after ADC conversion is complete, MCU exits sleep mode
//Note:kbd interrupt may wake MCU from sleep mode if MCUCR not set properly
MCUCR = SE + SM0;//disable other interrupts
//Start conversion by going into sleep mode
#asm
sleep
#endasm
//Read ADCL first then ADCH
ADC_value = ADCL;
ADC_value += ((unsigned int) ADCH << 8);
voltage = (float) ADC_value * aref / 1024.0;
MCUCR = MCUCR_save;//restore MCUCR                    
return voltage;
end    
   
//get_pressure stores the pressure output in str
void get_pressure(unsigned char*str)
begin
float Vina121, pressure;
Vina121 = ADC_get(PRESSURE_SENSOR_CHANNEL);
pressure /*mmHg*/ = Vina121 * 1856.4 / mpx2100_supply_V;
ftoa(pressure, 1, str);
strcatf(str,"mmHg");
end
                                       
//get_temperature stores the temperature in str
void get_temperature(unsigned char*str)        
begin
unsigned int temp, temp1, temp0;
temp = ds1820_temperature_10(0);//input arg=0, because only 1 sensor on bus
temp0 = temp%100;//digits after decimal point
temp1 = temp/100; //digits before decimal point
sprintf(str, "%2d.%02dC ",temp1,temp0);
//printf("%2d.%2dC \r",temp1,temp0);
end
    
//get_ADC(unsigned char*str,unsigned char channel) stores the channel inputs in str
void get_ADC(unsigned char*str, unsigned char channel)
begin
float adc_value;
adc_value = ADC_get(channel);
ftoa(adc_value,2,str);
strcatf(str," V");
end

interrupt [ADC_INT] void adc_done(void)
begin
end