//stv5730.c
//driver for stv5730 on screen decoder chip
#include "stv5730.h"
#include <delay.h>

#ifndef begin
#define begin {
#define end }
#endif       

//#define stv_debug 1 //decomment for debug

void STV_gotoxy(unsigned char x, unsigned char y)
begin
cursor_y = y;
cursor_x = x;
end    
   
//STV_monitor
//Checks the mute output of STV5730 to determine
//whether an input is present and switch between
//video full page mode and full page mode accordingly
void STV_monitor(void)
begin
if (STV_MUTE) //no signal detected
  begin//change to full page mode
  if (STV_current_mode == VIDEO)
    begin
    STV_current_mode = FULL_PAGE;
    STV_full_page_mode();
    end
  end                            
else
  begin//change to video full page mode
  if (STV_current_mode == FULL_PAGE)
    begin
    STV_current_mode = VIDEO;
    STV_video_mode();
    end
  end
end

#define BE (1<<3) //background enabled      
void STV_clrscr(void)
begin
unsigned char byte1, byte0;
unsigned int i;
byte1=0;
byte0=0;
STV_send_16(byte1, byte0);//move to 0,0
for (i=0;i<308;i++)
  begin
  byte1 = STV_WRITE;
  byte0 = 0x0b;//space
  STV_send_16(byte1, byte0);
  end
end            
                                                      
/*STV_printf
str - input string
color - text color
blink - 0, no blink, 1 blink
background - 0 no background for text, 1 background for text enabled
*/
void STV_printf(unsigned char flash*str, unsigned char color,unsigned char blink, unsigned char background)
begin
unsigned char byte1, byte0, code, char_count;
//Move to the correct position
byte1 = cursor_y;
byte0 = cursor_x;      
STV_send_16(byte1,byte0);//Set write pointer
byte1 = STV_WRITE + (background<<3) + color;            
char_count = 0;
while(*str!=0)
  begin
  code = char_code[*str];//convert to stv5730 char code
  str++;
  char_count++;
  if (code>=0x80)//(code==0xff)
    begin//invalid code
    #ifdef stv_debug
    printf("Err in STV_printf, invalid code\n\r");
    #endif
    end          
  else
    begin//Valid code, print to screen
    byte0 = (blink<<7) + code;
    STV_send_16(byte1,byte0);
    cursor_x++;
    if (cursor_x==28)
      begin
      cursor_x = 0;
      cursor_y++;
      if (cursor_y==12) cursor_y = 0;
      end
    end//else
  if (char_count>=308)
    begin
    //likely buffer overflow error
    #ifdef stv_debug
    printf("Err in STV_printf, buffer ovrfl\n\r");
    #endif                                        
    break;
    end
  end//while
end    

void STV_print(unsigned char *str, unsigned char color, unsigned char blink, unsigned char background)
begin
unsigned char byte1, byte0, code, char_count;
//Move to the correct position
byte1 = cursor_y;
byte0 = cursor_x;      
STV_send_16(byte1,byte0);//Set write pointer
byte1 = STV_WRITE + (background<<3) + color;            
char_count = 0;
while(*str!=0)
  begin
  code = char_code[*str];//convert to stv5730 char code
  str++;
  char_count++;
  if (code>=0x80)//code==0xff)
    begin//invalid code
    #ifdef stv_debug
    printf("Err in STV_print, invalid code\n\r");
    #endif
    end          
  else
    begin//Valid code, print to screen
    byte0 = (blink<<7) + code;
    STV_send_16(byte1,byte0);
    cursor_x++;
    if (cursor_x==28)
      begin
      cursor_x = 0;
      cursor_y++;
      if (cursor_y==12) cursor_y = 0;
      end
    end//else
  if (char_count>=308)
    begin
    //likely buffer overflow error
    #ifdef stv_debug
    printf("Err in STV_print, buffer ovrfl\n\r");
    #endif                                        
    break;
    end
  end//while 

end    
//STV_putcode - writes the code for STV5730A directly
void STV_putcode(unsigned char code, unsigned char color, unsigned char blink, unsigned char background)
begin
unsigned char byte1, byte0;
STV_send_16(cursor_y,cursor_x);
byte1 = STV_WRITE + (background<<3) + color;
byte0 = (blink<<7) + code;
STV_send_16(byte1,byte0);
cursor_x++;
if (cursor_x==28)
  begin
  cursor_x = 0;
  cursor_y++;
  if (cursor_y==12) cursor_y = 0;
  end
end
     
//STV_putch returns 1 if char written succesfully
unsigned char STV_putch(unsigned char c, unsigned char color, unsigned char blink, unsigned char background)
begin
unsigned char byte1, byte0, code;
//Move to the correct position
byte1 = cursor_y;
byte0 = cursor_x;      
STV_send_16(byte1,byte0);//Set write pointer
byte1 = STV_WRITE + (background<<3) + color;            
code = char_code[c];//convert to stv5730 char code
if (code>=0x80)//code==0xff)
  begin//invalid code
  #ifdef stv_debug
  printf("Err in STV_print, invalid code\n\r");
  #endif  
  return 0;
  end          
else
  begin//Valid code, print to screen
  byte0 = (blink<<7) + code;
  STV_send_16(byte1,byte0);
  cursor_x++;
  if (cursor_x==28)
    begin
    cursor_x = 0;
    cursor_y++;
    if (cursor_y==12) cursor_y = 0;
    end
  end//else
return 1;//succesful write
end//end STV_putch

#define WITH_BORDER (1<<6)
#define ROW_DISPLAY (1<<7)

void STV_initialize(void)
begin
unsigned char i, byte1, byte0, hpos_offset, vpos_offset;   
//initialize port directions and pull ups
//data, clk and csn are outputs
STV_DATA_DDR 	= 1;
STV_CLK_DDR 	= 1;
STV_CSN_DDR	= 1;
//mute is an input, with pull up off
STV_MUTE_DDR	= 0;
STV_MUTE_PORT	= 0;
//Idle conditions, clk = 0, data = 0
STV_CLK = 0; //not required
STV_DATA = 0;//not required

//Send init sequence   
STV_send_16(0x30,0x00);
STV_send_16(0x30,0x00);
STV_send_16(0x30,0x00);//extra
STV_send_16(0x30,0x00);//extra
delay_us(100);
STV_send_16(0x00,0xdb);
STV_send_16(0x10,0x00);
//Set control registers
//Write pointer format(16 bits)
//0000  BUF[11:8] STRU[7:6] 0 DEPL[4:0]
//For register access, set BUF[11:8] = 0,
//STRU = 11,(0xc0 for lower byte)
//DEPL =12, zoom register
//	13, color register
//	14, control reg
//	15, position reg
//	16, mode reg

//init write pointer to color register
byte1 = STV_REG_ADD_HI;//BUF = 0000
byte0 = STV_REG_ADD_LO_COLOR;
STV_send_16(byte1, byte0);

//Write pointer now at color reg
byte1 = STV_COLOR_HI;
byte0 = STV_COLOR_LO;
STV_send_16(byte1,byte0);

//control         
byte1 = STV_CONTROL_HI;
byte0 = STV_CONTROL_LO;
STV_send_16(byte1, byte0);

//pos
byte1 = STV_POS_HI;
byte0 = STV_POS_LO;
STV_send_16(byte1, byte0);

//mode
byte1 = STV_MODE_HI;
byte0 = STV_MODE_LO;
STV_send_16(byte1, byte0);   
STV_current_mode = FULL_PAGE;

//Set row attributes
//move write pointer
byte1 = 0x00;
byte0 = 0xc0;//address of row 0
STV_send_16(byte1, byte0); 
hpos_offset = HPOS_OFFSET;
vpos_offset = VPOS_OFFSET;       
for (i=0;i<11;i++)//write to row attrib for 0 to 10
  begin
  byte1 = 0x10 + hpos_offset;
  byte0 = WITH_BORDER + ROW_DISPLAY + vpos_offset;
  //byte0 = ROW_DISPLAY + vpos_offset;
  //printf("row attributes %x,%x\n\r",byte1,byte0);
  STV_send_16(byte1, byte0);
  end//for
                   
STV_clrscr();

#ifdef stv_debug
//STV_echo_settings();
printf("STV5730 init reg complete\n\r");
#endif

end    

//Change to video input mode              
void STV_video_mode(void)
begin
unsigned char byte1,byte0;
//write to control and mode registers
//init write pointer to control register
byte1 = STV_REG_ADD_HI;
byte0 = STV_REG_ADD_LO_CONTROL;
STV_send_16(byte1, byte0);    
byte1 = STV_CONTROL_HI_VIDEO;
byte0 = STV_CONTROL_LO_VIDEO;
STV_send_16(byte1, byte0);
//Write to mode register
byte1 = STV_REG_ADD_HI;
byte0 = STV_REG_ADD_LO_MODE;
STV_send_16(byte1, byte0);
byte1 = STV_MODE_HI_VIDEO;
byte0 = STV_MODE_LO_VIDEO;
STV_send_16(byte1, byte0);
#ifdef debug
printf("STV in Video input mode\n\r");
#endif
end
                                          
//Change to full page mode(no video input)      
void STV_full_page_mode(void)                
begin     
unsigned char byte1, byte0;                                   
//write to control and mode registers 
byte1 = STV_REG_ADD_HI;
byte0 = STV_REG_ADD_LO_CONTROL;
STV_send_16(byte1, byte0);    
byte1 = STV_CONTROL_HI;
byte0 = STV_CONTROL_LO;
STV_send_16(byte1, byte0);
//Write to mode register
byte1 = STV_REG_ADD_HI;
byte0 = STV_REG_ADD_LO_MODE;
STV_send_16(byte1, byte0);
byte1 = STV_MODE_HI;
byte0 = STV_MODE_LO;
STV_send_16(byte1, byte0);
#ifdef stv_debug
printf("STV in full page mode\n\r");
#endif
end

//Send a 16 bit msg to the STV5730
//byte1 is the higher byte, byte0 is the lower byte
void STV_send_16(unsigned char byte1, unsigned char byte0)
begin
unsigned char data, i;
unsigned int j=1;
while(kbd_edge_count!=0)
  begin
  j++;
  if (j==0) break;//kbd_edge_count not moving
  end//  printf("<%d>",kbd_edge_count); //wait for kbd transmission complete
kbd_cts_off();
/*
#asm
cli
#endasm
//take kbd_clk low, to prevent transfer
GIMSK = GIMSK&0xbf;//disable falling edge interrupt
kbd_clk_port = 0;//pull-up off
kbd_clk_dir = 1;//set as output low
*/

STV_CLK = 1;
STV_CSN = 0;
//Data is sampled on the rising edge, send msb first
data = byte1;
for(i=7;i<8;i--)//i is an unsigned char, no negative
  begin
  STV_CLK = 0;
  STV_DATA = (data >> i) & 0x01;
  STV_CLK = 1;//data is latched on rising edge
  #asm
  nop
  nop
  #endasm   
  end       
data = byte0;//send lower byte
for(i=7;i<8;i--)
  begin
  STV_CLK = 0;
  STV_DATA = (data >> i) & 0x01;
  STV_CLK = 1;//data is latched on rising edge
  #asm
  nop
  nop
  #endasm  
  end
STV_CSN = 1;
STV_CLK = 1;
//next msg must be sent at least 4*maxHZ us later.
//delay_us(16);//no errors observed wuth this disabled

//relase kbd_clk so that kbd can continue transfer
/*
GIMSK = GIMSK|0x40;//enable falling edge interrupt
kbd_clk_port = 1;//pull-up on
kbd_clk_dir = 0;//set as input
#asm
sei
#endasm                       
*/
kbd_cts_on();

end      


void STV_send_8(unsigned char byte0)
begin
char i, data;
STV_CLK = 1;
STV_CSN = 0;
//Data is sampled on the rising edge, send msb first
data = byte0;
for(i=7;i>=0;i--)
  begin
  STV_CLK = 0;
  STV_DATA = (data >> i) & 0x01;
  STV_CLK = 1;//data is latched on rising edge
  #asm
  nop
  nop
  #endasm  
  end
STV_CSN = 1;
STV_CLK = 1;
//next msg must be sent at least 4*maxHZ us later.
end                                                                                              

//*******TESTING**********************************
void STV_high_imp(void)
begin
unsigned char byte1,byte0;
printf("high imp");
byte1 = STV_REG_ADD_HI;
byte0 = STV_REG_ADD_LO_MODE;
STV_send_16(byte1,byte0);
byte1 = STV_MODE_HI;
byte0 = STV_MODE_LO;
STV_send_16(byte1,byte0);  
end

void STV_low_imp(void)
begin
unsigned char byte1,byte0;
printf("low imp");
byte1 = STV_REG_ADD_HI;
byte0 = STV_REG_ADD_LO_MODE;
STV_send_16(byte1,byte0);
byte1 = STV_MODE_HI_VIDEO;
byte0 = STV_MODE_LO_VIDEO;
STV_send_16(byte1,byte0);
end

void STV_set_mode_reg(void)
begin
STV_send_16(STV_REG_ADD_HI,STV_REG_ADD_LO_MODE);
STV_send_16(STV_MODE_HI,STV_MODE_LO);
end

void STV_set_mode_reg2(void)
begin
STV_send_16(STV_REG_ADD_HI,STV_REG_ADD_LO_MODE);
STV_send_16(0b00010001,0b10010100);//mute on
end                   

void STV_echo_settings(void)
begin
printf("MODE REG %02x,%02x",STV_MODE_HI,STV_MODE_LO);
printf(" VIDEO %02x,%02x\n\r",STV_MODE_HI_VIDEO,STV_MODE_LO_VIDEO);
end