//stv5730.h  
//kbd   
#ifndef kbd_clk_port 
#define kbd_clk_port  PORTD.2
#define kbd_data_port PORTD.3
#define kbd_clk_pin   PIND.2
#define kbd_data_pin  PIND.3
#define kbd_clk_dir   DDRD.2
#define kbd_data_dir  DDRD.3
#endif
//end kbd
                 
//Variables
unsigned char cursor_x, cursor_y;
           
/* Port allocations are as follows */
#define STV_DATA 	PORTD.4
#define STV_DATA_DDR    DDRD.4
#define STV_CLK 	PORTD.5
#define STV_CLK_DDR	DDRD.5
#define STV_CSN		PORTD.6
#define STV_CSN_DDR	DDRD.6
#define STV_MUTE	PIND.7
#define STV_MUTE_PORT	PORTD.7
#define STV_MUTE_DDR	DDRD.7

#define FULL_PAGE 0
#define VIDEO	  1
unsigned char STV_current_mode;   //Video or Full Page

//Colors
#ifndef BLACK
#define BLACK	0
#define BLUE	1
#define GREEN	2
#define CYAN	3
#define RED	4
#define MAGENTA	5
#define YELLOW	6
#define WHITE 	7
#endif

#define STV_REG_ADD_HI 		0x00
#define STV_REG_ADD_LO_ZOOM	0xc0 + 12
#define STV_REG_ADD_LO_COLOR	0xc0 + 13
#define STV_REG_ADD_LO_CONTROL	0xc0 + 14
#define STV_REG_ADD_LO_POS	0xc0 + 15
#define STV_REG_ADD_LO_MODE	0xc0 + 16

#define HPOS_OFFSET 0
//VPOS 0x00-0x11 (0 to 17) 0x20-0x31(-1 to -17)
//positive vpos, shifts text down
#define VPOS_OFFSET 0               
                    
#define STV_WRITE	0x10//hi-byte must contain this for write
#define SCREEN_BACKGROUND 	BLACK
#define CHAR_BORDER		BLACK
#define CHAR_BACKGROUND		YELLOW//BLACK
#define STV_COLOR_HI	STV_WRITE + (SCREEN_BACKGROUND<<1) + (CHAR_BORDER>>2)
#define STV_COLOR_LO	(CHAR_BORDER<<6) + CHAR_BACKGROUND

#define NORMAL_FULL_PAGE 0x08 	//Normal full page mode C11 = 1, Video f. p. m. C11 = 0
#define EXT_LUM		 0x04 	//Ext luminance
#define CHAR_COLOR_EN	 0x02	//Char color encoding enabled
#define NTSC		 0x00	//NTSC mode                                 

//Without Video input signal
#define STV_CONTROL_HI 		STV_WRITE + NORMAL_FULL_PAGE + CHAR_COLOR_EN + NTSC

//With video input signal
#define STV_CONTROL_HI_VIDEO 	STV_WRITE + CHAR_COLOR_EN + NTSC

#define INTERNAL_SYNC	0x80
#define NO_BLINK	0x00
#define BLINK_75_100	(0x01 << 5)
#define BLINK_50_100	(0x02 << 5)
#define BLINK_25_100	(0x03 << 5)
#define ONE_S_BLINK	0x00
#define HALF_S_BLINK	(0x01 << 4)
#define DISPLAY_ON	0x04
#define CHAR_BACKGROUND_DISABLED 0x02
//In mixed mode, a video signal must be provided
#define FULL_PAGE_MODE	0x01                    
//In full page mode, the chip generates it's own video sync. Video output is non-interlaced.                   

//Without Video input signal
#define STV_CONTROL_LO 	INTERNAL_SYNC + BLINK_50_100 + ONE_S_BLINK + DISPLAY_ON + FULL_PAGE_MODE        
//With Video input signal                                                                
#define STV_CONTROL_LO_VIDEO 	INTERNAL_SYNC + BLINK_50_100 + ONE_S_BLINK + DISPLAY_ON

#define VERTICAL_POS 	20 //Max 63
#define HORIZONTAL_POS 	30 //Max 63
#define STV_POS_HI	STV_WRITE + (VERTICAL_POS >> 2)
#define STV_POS_LO	(VERTICAL_POS << 6) + HORIZONTAL_POS

#define VOUT2_HIGH_IMP 		0x00
#define VOUT2_LOW_IMP  		0x01
//Without video input
#define STV_MODE_HI  		0x11//STV_WRITE + VOUT2_HIGH_IMP    
//With video input
#define STV_MODE_HI_VIDEO 	0x10//STV_WRITE + VOUT2_LOW_IMP
                               
#define VOUT2_OUTPUT_VIN 	0x00
#define VOUT2_OUTPUT_CSYNC 	0x80
#define COLOR_EN 		0x00
#define HIGH_GAIN       	0x20
#define REINSERT_SYNC  		0x10
#define MUTE_TIME_CONST_32	0x08
#define MUTE_ON			0x04
#define DETECT_MISSING_SYNC 	0x02
#define BAR_EN			0x01
#define STV_MODE_LO		MUTE_ON + REINSERT_SYNC + HIGH_GAIN
#define STV_MODE_LO_VIDEO 	STV_MODE_LO
                                                       
//Adapted from http://www.neurophys.wisc.edu/www/comp/docs/ascii.html
unsigned char eeprom char_code[] = {//lookup table for stv5730
0xff,	//NUL    (Null char.)
0xff,	//SOH    (Start of Header)
0xff,	//STX    (Start of Text)
0xff,	//ETX    (End of Text)
0xff,	//EOT    (End of Transmission)
0xff,	//ENQ    (Enquiry)
0xff,	//ACK    (Acknowledgment)
0xff,	//BEL    (Bell)
0xff,	// BS    (Backspace)
0xff,	// HT    (Horizontal Tab)
0xff,	// LF    (Line Feed)
0xff,	// VT    (Vertical Tab)
0xff,	// FF    (Form Feed)
0xff,	// CR    (Carriage Return)
0xff,	// SO    (Shift Out)
0xff,	// SI    (Shift In)
0xff,	//DLE    (Data Link Escape)
0xff,	//DC1 	 (XON) (Device Control 1)
0xff,	//DC2    (Device Control 2)
0xff,	//DC3 	 (XOFF)(Device Control 3)
0xff,	//DC4    (Device Control 4)
0xff,	//NAK    (Negative Acknowledgement)
0xff,	//SYN    (Synchronous Idle)
0xff,	//ETB   0x0b,	 (End of Trans. Block)
0xff,	//CAN    (Cancel)
0xff,	// EM    (End of Medium)
0xff,	//SUB    (Substitute)
0xff,	//ESC    (Escape)
0xff,	//FS    (File Separator)
0xff,	//GS    (Group Separator)
0xff,	//RS    (Request to Send)(Record Separator)
0xff,	//US    (Unit Separator)
0x0b,	//SP    (Space)
0xff,	//!
0x29,	//"
0xff,	//#
0xff,	//$
0xff,	//%
0xff,	//&
0x29,	//'
0xff,	//(
0xff,	//)
0x5f,	//*
0x5e,	//+
0xff,	//,
0x0a,	//-
0x27,	//.
0x28,	///
0x00,	//0
0x01,	//1
0x02,	//2
0x03,	//3
0x04,	//4
0x05,	//5
0x06,	//6
0x07,	//7
0x08,	//8
0x09,	//9
0x26,	//:
0xff,	//;
0xff,	//<
0x78,	//=
0xff,	//>
0x70,	//?
0xff,	//@
0x0c,	//A
0x0d,	//B
0x0e,	//C
0x0f,	//D
0x10,	//E
0x11,	//F
0x12,	//G
0x13,	//H
0x14,	//I
0x15,	//J
0x16,	//K
0x17,	//L
0x18,	//M
0x19,	//N
0x1a,	//O
0x1b,	//P
0x1c,	//Q
0x1d,	//R
0x1e,	//S
0x1f,	//T
0x20,	//U
0x21,	//V
0x22,	//W
0x23,	//X
0x24,	//Y
0x25,	//Z
0xff,	//[
0xff,	//\
0xff,	//]
0xff,	//^
0x72,	//_
0x29,	//`
0x2a,	//a
0x2b,	//b
0x2c,	//c
0x2d,	//d
0x2e,	//e
0x2f,	//f
0x30,	//g
0x31,	//h
0x32,	//i
0x33,	//j
0x34,	//k
0x35,	//l
0x36,	//m
0x37,	//n
0x38,	//o
0x39,	//p
0x3a,	//q
0x3b,	//r
0x3c,	//s
0x3d,	//t
0x3e,	//u
0x3f,	//v
0x40,	//w
0x41,	//x
0x42,	//y
0x43,	//z
0xff,	//{
0xff,	//|
0xff,	//}
0xff,	//~
0xff	//DEL 0x7f'th element
};//char_code
//Testing
void STV_echo_settings(void);